/*
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 * 
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *          http://www.eclipse.org/legal/epl-v10.html
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.geoviewer;

import com.vividsolutions.jts.geom.*;
import com.vividsolutions.jts.geom.impl.CoordinateArraySequence;
import java.beans.PropertyChangeListener;
import java.util.concurrent.Callable;
import java.util.logging.Logger;
import org.geotools.coverage.grid.GridCoverage2D;
import org.geotools.geometry.Envelope2D;
import org.geotools.geometry.jts.JTSFactoryFinder;
import org.openide.util.Exceptions;

/**
 * Base class for GridCoverage2D Geographical objects (Grids) Geometric
 * functions getCentroid() and getBoundingBox() are implemented.
 *
 *
 *
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
public abstract class Grid implements Geographical<GridCoverage2D> {

    private GridCoverage2D internalGrid;
    private Callable<GridCoverage2D> callable;
    private Geometry boundingBox;
    private Point centroid;
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.geoviewer.Grid");

    /**
     * Initialize the Grid with a GridCoverage2D Use {@link Grid#Grid(java.util.concurrent.Callable}
     * to postpone grid initialization.
     *
     * @param grid
     */
    public Grid(GridCoverage2D grid) {
        this.internalGrid = grid;
    }

    /**
     * Initialize the Grid with a Callable which returns a GridCoverage2D on
     * call. Grid initialization can take some time. This way initialization can
     * be postponed to the moment it is needed to avoid unnecessary
     * calculations.
     *
     * @param callable
     */
    public Grid(Callable<GridCoverage2D> callable) {
        this.callable = callable;
    }

    private GridCoverage2D getInternalGrid() {
        if (internalGrid == null && callable == null) {
            throw new IllegalStateException("Either the internal grid or the future should not be null");
        }
        if (internalGrid == null) {
            try {
                internalGrid = callable.call();
            } catch (Exception ex) {
                Exceptions.printStackTrace(ex);
            }
        }
        return internalGrid;
    }

    @Override
    public Point getCentroid() {
        if (getInternalGrid() == null) {
            LOGGER.warning("FIXME:getCentroid called while grid is null");
            return null;
        }

        if (centroid == null) {
            Double x = getInternalGrid().getEnvelope2D().getCenterX();
            Double y = getInternalGrid().getEnvelope2D().getCenterY();
            centroid = new Point(new CoordinateArraySequence(new Coordinate[]{new Coordinate(x, y)}), JTSFactoryFinder.getGeometryFactory(null));
        }
        return centroid;
    }

    @Override
    public GridCoverage2D getRenderObject(Envelope envelope) {
        return getInternalGrid();
    }

    @Override
    public Geometry getBoundingBox() {
        if (getInternalGrid() == null) {
            LOGGER.warning("FIXME:getBoundingBox called while grid is null");
            return null;
        }

        if (boundingBox == null) {
            LOGGER.finest("Initializing bounding box for Grid");
            Coordinate[] coordinates = new Coordinate[5];

            Envelope2D e2d = getInternalGrid().getEnvelope2D();

            coordinates[0] = new Coordinate(e2d.x, e2d.y);
            coordinates[1] = new Coordinate(e2d.x + e2d.width, e2d.y);
            coordinates[2] = new Coordinate(e2d.x + e2d.width, e2d.y + e2d.height);
            coordinates[3] = new Coordinate(e2d.x, e2d.y + e2d.height);
            coordinates[4] = new Coordinate(e2d.x, e2d.y);

            GeometryFactory geometryFactory = JTSFactoryFinder.getGeometryFactory(null);
            LinearRing linearRing = new LinearRing(new CoordinateArraySequence(coordinates), geometryFactory);
            boundingBox = new Polygon(linearRing, null, geometryFactory);
        }
        return boundingBox;
    }

    @Override
    public String getIconLabel() {
        return null;
    }

    @Override
    public String getIconPath() {
        return null;
    }

    @Override
    public String getTooltipText() {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void addPropertyChangeListener(PropertyChangeListener listener) {
    }

    @Override
    public void removePropertyChangeListener(PropertyChangeListener listener) {
    }
}
